from django.forms import widgets as dwidgets


class InputMixin:
    template_name = "pretty/widgets/input.html"

    def __init__(self, attrs=None):
        if attrs is None:
            attrs = {"class": "input"}
        else:
            if "class" in attrs:
                attrs["class"] = "input" + " " + attrs["class"]
            else:
                attrs["class"] = "input"
        super().__init__(attrs=attrs)


class CheckboxMixin:
    template_name = "pretty/widgets/checkbox.html"

    def __init__(self, attrs=None):
        if attrs is None:
            attrs = {"class": "checkbox"}
        else:
            if "class" in attrs:
                attrs["class"] = "checkbox" + " " + attrs["class"]
            else:
                attrs["class"] = "checkbox"
        super().__init__(attrs=attrs)


class CheckboxInput(CheckboxMixin, dwidgets.CheckboxInput):
    template_name = "pretty/widgets/checkbox.html"


class TextInput(InputMixin, dwidgets.TextInput):
    template_name = "pretty/widgets/input.html"


class NumberInput(InputMixin, dwidgets.NumberInput):
    template_name = "pretty/widgets/number.html"


class EmailInput(InputMixin, dwidgets.EmailInput):
    template_name = "pretty/widgets/email.html"


class URLInput(InputMixin, dwidgets.URLInput):
    template_name = "pretty/widgets/url.html"


class PasswordInput(InputMixin, dwidgets.PasswordInput):
    template_name = "pretty/widgets/password.html"


class HiddenInput(InputMixin, dwidgets.HiddenInput):
    template_name = "pretty/widgets/hidden.html"


class MultipleHiddenInput(HiddenInput, dwidgets.HiddenInput):
    template_name = "pretty/widgets/hidden.html"


class FileInput(InputMixin, dwidgets.FileInput):
    template_name = "pretty/widgets/file.html"


class ClearableFileInput(InputMixin, dwidgets.ClearableFileInput):
    template_name = "pretty/widgets/file.html"


class Textarea(dwidgets.Textarea):
    template_name = "pretty/widgets/textarea.html"

    def __init__(self, attrs=None):
        if attrs is None:
            attrs = {"class": "textarea"}
        else:
            if "class" in attrs:
                attrs["class"] = "textarea" + " " + attrs["class"]
            else:
                attrs["class"] = "textarea"
        super().__init__(attrs=attrs)


class DateTimeBaseInput(InputMixin, dwidgets.TextInput):
    pass


class DateInput(DateTimeBaseInput, dwidgets.DateInput):
    pass


class DateTimeInput(DateTimeBaseInput, dwidgets.DateTimeInput):
    pass


class TimeInput(DateTimeBaseInput, dwidgets.TimeInput):
    pass


class Select(dwidgets.Select):
    template_name = "pretty/widgets/select.html"
    option_template_name = "pretty/widgets/select_option.html"


class SelectMultiple(dwidgets.SelectMultiple):
    template_name = "pretty/widgets/select_multiple.html"
    option_template_name = "pretty/widgets/select_option.html"


class RadioSelect(dwidgets.RadioSelect):
    template_name = "pretty/widgets/radio.html"
    option_template_name = "pretty/widgets/radio_option.html"


class CheckboxSelectMultiple(dwidgets.CheckboxSelectMultiple):
    template_name = "pretty/widgets/checkbox_select.html"
    option_template_name = "pretty/widgets/checkbox_option.html"


class MultiWidget(dwidgets.MultiWidget):
    template_name = "pretty/widgets/multiwidget.html"


class SplitDateTimeWidget(dwidgets.SplitDateTimeWidget):
    template_name = "pretty/widgets/splitdatetime.html"


class SplitHiddenDateTimeWidget(dwidgets.SplitHiddenDateTimeWidget):
    template_name = "pretty/widgets/splithiddendatetime.html"


class SelectDateWidget(dwidgets.SelectDateWidget):
    template_name = "pretty/widgets/select_date.html"
